@@ -164,14 +164,13 @@ void commissioning(){
 // 声明参数自整定结构体变量
 struct CommissioningDataStruct COMM;
 int16 bool_single_phase_excitation = FALSE;
-// int16 G.bool_comm_status = FALSE;
+// int16 COMM.bool_comm_status = FALSE;
 int16 bool_tuning = TRUE;
 int16 bool_CL_tuned = FALSE;
 int16 bool_VL_tuned = FALSE;
 int16 bool_PL_tuned = FALSE;
 
 
-#define EXCITE_BETA_AXIS_AND_MEASURE_PHASE_B TRUE
 #define UD_OUTPUT CTRL.O->uab_cmd[0]
 #define UQ_OUTPUT CTRL.O->uab_cmd[1]
 
@@ -193,8 +192,8 @@ void init_COMM(){
     COMM.bool_collecting = FALSE;
     int i;
     for(i=0;i<COMM_IV_SIZE_R1;++i){
-        COMM.i_data_R1[i] = 0.0;
-        COMM.v_data_R1[i] = 0.0;
+        COMM.i_phase_data_R1[i] = 0.0;
+        COMM.v_phase_data_R1[i] = 0.0;
     }
     for(i=0;i<COMM_IV_SIZE_L1;++i){
         COMM.i_data_L1[i] = 0.0;
@@ -211,7 +210,7 @@ void init_COMM(){
     COMM.number_of_repeats_Js = 0.0;
 
     // global status vairbale
-    G.bool_comm_status = 1;
+    COMM.bool_comm_status = 1;
 
     COMM.counterEntered = 0;
     COMM.i = 0;
@@ -361,10 +360,6 @@ void COMM_resistanceId(REAL id_fb, REAL iq_fb){
         UQ_OUTPUT = 0.0;
     #endif
 
-    // for ParkSul2012
-    CTRL.I->idq_cmd[0] = REGULATOR.Ref;
-    CTRL.I->idq_cmd[1] = 0.0;
-
     // collect steady state data
     if(COMM.bool_collecting){
         #if EXCITE_BETA_AXIS_AND_MEASURE_PHASE_B
@@ -377,10 +372,10 @@ void COMM_resistanceId(REAL id_fb, REAL iq_fb){
         COMM.counterSS += 1;
 
         if(COMM.counterSS>800){
-            COMM.i_data_R1[COMM.i] = COMM.current_sum/(REAL)COMM.counterSS * -SIN_DASH_2PI_SLASH_3; // beta -> phase b
-            COMM.v_data_R1[COMM.i] = COMM.voltage_sum/(REAL)COMM.counterSS * -SIN_DASH_2PI_SLASH_3; // beta -> phase b
+            COMM.i_phase_data_R1[COMM.i] = COMM.current_sum/(REAL)COMM.counterSS * -SIN_DASH_2PI_SLASH_3; // beta -> phase b
+            COMM.v_phase_data_R1[COMM.i] = COMM.voltage_sum/(REAL)COMM.counterSS * -SIN_DASH_2PI_SLASH_3; // beta -> phase b
             #if PC_SIMULATION
-                printf("%f, %f\n", COMM.i_data_R1[COMM.i], COMM.v_data_R1[COMM.i]);
+                printf("%f, %f\n", COMM.i_phase_data_R1[COMM.i], COMM.v_phase_data_R1[COMM.i]);
             #endif
             COMM.bool_collecting = FALSE;
             ++COMM.i;
@@ -392,7 +387,7 @@ void COMM_resistanceId(REAL id_fb, REAL iq_fb){
         COMM.counterSS = 0;
 
         // check steady state and assign boolean variable
-        if(reachSteadyStateCurrent(pid1_id.Ref-pid1_id.Fbk, MOTOR_RATED_CURRENT_RMS)){
+        if(reachSteadyStateCurrent(REGULATOR.Ref-REGULATOR.Fbk, MOTOR_RATED_CURRENT_RMS)){
 
             COMM.bool_collecting = TRUE;
 
@@ -400,10 +395,10 @@ void COMM_resistanceId(REAL id_fb, REAL iq_fb){
 
                 // Get resistance value
                 if(FALSE){
-                    COMM.R = (COMM.v_data_R1[1] - COMM.v_data_R1[0]) / (COMM.i_data_R1[1] - COMM.i_data_R1[0]);
+                    COMM.R = (COMM.v_phase_data_R1[1] - COMM.v_phase_data_R1[0]) / (COMM.i_phase_data_R1[1] - COMM.i_phase_data_R1[0]);
                 }else{
                     REAL m,b,r;
-                    if(linreg(( int)(0.3*RS_ID_NUMBER_OF_STAIRS), COMM.i_data_R1, COMM.v_data_R1, &m, &b, &r)){
+                    if(linreg(( int)(0.3*RS_ID_NUMBER_OF_STAIRS), COMM.i_phase_data_R1, COMM.v_phase_data_R1, &m, &b, &r)){
                         #if PC_SIMULATION
                             printf("FAILURE\n");
                         #endif
@@ -416,11 +411,11 @@ void COMM_resistanceId(REAL id_fb, REAL iq_fb){
                     int j;
                     printf("---CURRENT LIST:\n");
                     for(j=0;j<COMM_IV_SIZE_R1;++j){
-                        printf("\t%g,\n", COMM.i_data_R1[j]);
+                        printf("\t%g,\n", COMM.i_phase_data_R1[j]);
                     }
                     printf("---VOLTAGE LIST:\n");
                     for(j=0;j<COMM_IV_SIZE_R1;++j){
-                        printf("\t%g,\n", COMM.v_data_R1[j]);
+                        printf("\t%g,\n", COMM.v_phase_data_R1[j]);
                     }
                 #endif
                 #if EXCITE_BETA_AXIS_AND_MEASURE_PHASE_B
@@ -428,7 +423,7 @@ void COMM_resistanceId(REAL id_fb, REAL iq_fb){
                 #else
                     COMM.last_voltage_command = UD_OUTPUT;
                 #endif
-                G.bool_comm_status = 2;
+                COMM.bool_comm_status = 2;
                 COMM.counterEntered = 0;
 
                 // UD_OUTPUT = 0.0;
@@ -448,38 +443,52 @@ void COMM_resistanceId_v2(REAL id_fb, REAL iq_fb){
     if(COMM.counterEntered<=1){
         // Current loop tuning (LL, RR, BW_c, delta, JJ, KE, npp)
         COMM_PI_tuning(PMSM_D_AXIS_INDUCTANCE, PMSM_RESISTANCE, 2*3.1415926*CL_TS_INVERSE * 0.025, 20, MOTOR_SHAFT_INERTIA, MOTOR_BACK_EMF_CONSTANT, MOTOR_NUMBER_OF_POLE_PAIRS);
-            // COMM_PI_tuning(7e-3, 0.8, 2*3.1415926*CL_TS_INVERSE * 0.025, 20, MOTOR_SHAFT_INERTIA, MOTOR_BACK_EMF_CONSTANT, MOTOR_NUMBER_OF_POLE_PAIRS);
+     // COMM_PI_tuning(7e-3, 0.8, 2*3.1415926*CL_TS_INVERSE * 0.025, 20, MOTOR_SHAFT_INERTIA, MOTOR_BACK_EMF_CONSTANT, MOTOR_NUMBER_OF_POLE_PAIRS);
     }
 
-    #define RS_ID_NUMBER_OF_STAIRS (COMM_IV_SIZE_R1*0.5-3)
-    // #define RS_ID_NUMBER_OF_STAIRS 40
+    // number of stairs (positive or negative current)
+    #define NOS_II (29)
+    #define NOS    (COMM_IV_SIZE_R1/2-1-NOS_II) // 200/2-30 = 70
+    #define RS_ID_MAXIMUM_CURRENT (1.414/0.8660254*MOTOR_RATED_CURRENT_RMS) /* [A] This specifies beta-axis current, so we need to make it larger so that the phase current is large enough This is motor related (to identify accurate resistance) */
     #ifdef _XCUBE1
-        #define RS_ID_MAXIMUM_CURRENT (1.0) /* [A] This is inverter related */ // *(REAL)MOTOR_RATED_CURRENT_RMS
+        #define RS_ID_LOW_CURRENT (0.3)
     #else
-        #define RS_ID_MAXIMUM_CURRENT (3.0) /* [A] This is inverter related */ 
+        #define RS_ID_LOW_CURRENT (1.0) /* [A] This is inverter related */ 
     #endif
-    REAL current_increase_per_step = RS_ID_MAXIMUM_CURRENT / (REAL)RS_ID_NUMBER_OF_STAIRS;
+    // REAL current_increase_per_step_equalStepSize = RS_ID_MAXIMUM_CURRENT / (REAL)NOS;
+    REAL Delta_I_Low  = RS_ID_LOW_CURRENT / (REAL)NOS;
+    REAL Delta_I_High = (RS_ID_MAXIMUM_CURRENT-RS_ID_LOW_CURRENT) / (REAL)NOS_II;
 
-    if(COMM.i>=RS_ID_NUMBER_OF_STAIRS){
-        COMM.current_command = -RS_ID_MAXIMUM_CURRENT;
-        COMM.current_command += current_increase_per_step*(COMM.i-RS_ID_NUMBER_OF_STAIRS);
-    }else{
+    if(COMM.i < NOS_II){
+        /* 正电流-高电流段 */
         COMM.current_command = RS_ID_MAXIMUM_CURRENT;
-        COMM.current_command -= current_increase_per_step*COMM.i;        
+        COMM.current_command -= Delta_I_High*COMM.i;
+    }else if(COMM.i < NOS+NOS_II){
+        /* 正电流-低电流段 */
+        COMM.current_command = RS_ID_MAXIMUM_CURRENT;
+        COMM.current_command -= Delta_I_High* NOS_II \
+                              + Delta_I_Low * (COMM.i-NOS_II);
+    }else if(COMM.i < NOS+2*NOS_II){
+        /* 负电流-高电流段 */
+        COMM.current_command = -RS_ID_MAXIMUM_CURRENT;
+        COMM.current_command += Delta_I_High*(COMM.i-NOS-NOS_II);
+    }else{
+        /* 负电流-低电流段 */
+        COMM.current_command = -RS_ID_MAXIMUM_CURRENT;
+        COMM.current_command += Delta_I_High* NOS_II \
+                              + Delta_I_Low * (COMM.i-NOS-2*NOS_II);
     }
 
     #if EXCITE_BETA_AXIS_AND_MEASURE_PHASE_B
         #define REGULATOR pid1_iq
         #define FEEDBACK  iq_fb
-        // pid1_id.Ref = 0.0;
     #else
         #define REGULATOR pid1_id
         #define FEEDBACK  id_fb
-        // pid1_iq.Ref = 0.0;
     #endif
 
     REGULATOR.Fbk = FEEDBACK;
-    REGULATOR.Ref = COMM.current_command; /* 改为从正到负 */
+    REGULATOR.Ref = COMM.current_command;
     if(G.FLAG_TUNING_CURRENT_SCALE_FACTOR)
         REGULATOR.Ref = 3;
     REGULATOR.calc(&REGULATOR);
@@ -487,14 +496,18 @@ void COMM_resistanceId_v2(REAL id_fb, REAL iq_fb){
     #if EXCITE_BETA_AXIS_AND_MEASURE_PHASE_B
         UD_OUTPUT = 0.0;
         UQ_OUTPUT = REGULATOR.Out;
+
+        // for ParkSul2012
+        CTRL.I->idq_cmd[0] = 0.0;
+        CTRL.I->idq_cmd[1] = REGULATOR.Ref;
     #else
         UD_OUTPUT = REGULATOR.Out;
         UQ_OUTPUT = 0.0;
-    #endif
 
-    // for ParkSul2012
-    CTRL.I->idq_cmd[0] = REGULATOR.Ref;
-    CTRL.I->idq_cmd[1] = 0.0;
+        // for ParkSul2012
+        CTRL.I->idq_cmd[0] = REGULATOR.Ref;
+        CTRL.I->idq_cmd[1] = 0.0;
+    #endif
 
     // collect steady state data
     if(COMM.bool_collecting){
@@ -508,10 +521,10 @@ void COMM_resistanceId_v2(REAL id_fb, REAL iq_fb){
         COMM.counterSS += 1;
 
         if(COMM.counterSS>800){
-            COMM.i_data_R1[COMM.i] = COMM.current_sum/(REAL)COMM.counterSS * -SIN_DASH_2PI_SLASH_3;
-            COMM.v_data_R1[COMM.i] = COMM.voltage_sum/(REAL)COMM.counterSS * -SIN_DASH_2PI_SLASH_3;
+            COMM.i_phase_data_R1[COMM.i] = COMM.current_sum/(REAL)COMM.counterSS * -SIN_DASH_2PI_SLASH_3;
+            COMM.v_phase_data_R1[COMM.i] = COMM.voltage_sum/(REAL)COMM.counterSS * -SIN_DASH_2PI_SLASH_3;
             #if PC_SIMULATION
-                printf("%f, %f\n", COMM.i_data_R1[COMM.i], COMM.v_data_R1[COMM.i]);
+                printf("I=%f, U=%f\n", COMM.i_phase_data_R1[COMM.i], COMM.v_phase_data_R1[COMM.i]);
             #endif
             COMM.bool_collecting = FALSE;
             ++COMM.i;
@@ -523,19 +536,22 @@ void COMM_resistanceId_v2(REAL id_fb, REAL iq_fb){
         COMM.counterSS = 0;
 
         // check steady state and assign boolean variable
-        if(reachSteadyStateCurrent(pid1_id.Ref-pid1_id.Fbk, MOTOR_RATED_CURRENT_RMS)){
+        if(reachSteadyStateCurrent(REGULATOR.Ref-REGULATOR.Fbk, RS_ID_MAXIMUM_CURRENT)){
 
             COMM.bool_collecting = TRUE;
 
+            /* 判断是否结束 */
             // if(fabs(COMM.current_command) > RS_ID_MAXIMUM_CURRENT){
-            if(COMM.i>=RS_ID_NUMBER_OF_STAIRS*2){
+            if(COMM.i>=NOS*2+NOS_II*2){
 
                 // Get resistance value
                 if(FALSE){
-                    COMM.R = (COMM.v_data_R1[1] - COMM.v_data_R1[0]) / (COMM.i_data_R1[1] - COMM.i_data_R1[0]);
+                    /* use two points */
+                    COMM.R = (COMM.v_phase_data_R1[1] - COMM.v_phase_data_R1[0]) / (COMM.i_phase_data_R1[1] - COMM.i_phase_data_R1[0]);
                 }else{
+                    /* least square fitting */
                     REAL m,b,r;
-                    if(linreg(( int)(0.3*RS_ID_NUMBER_OF_STAIRS), COMM.i_data_R1, COMM.v_data_R1, &m, &b, &r)){
+                    if(linreg(( int)(1.0*NOS_II), COMM.i_phase_data_R1, COMM.v_phase_data_R1, &m, &b, &r)){
                         #if PC_SIMULATION
                             printf("FAILURE\n");
                         #endif
@@ -548,19 +564,21 @@ void COMM_resistanceId_v2(REAL id_fb, REAL iq_fb){
                     int j;
                     printf("---PHASE B CURRENT LIST:\n");
                     for(j=0;j<COMM_IV_SIZE_R1;++j){
-                        printf("\t%g,\n", COMM.i_data_R1[j] );
+                        printf("\t%g,\n", COMM.i_phase_data_R1[j] );
                     }
                     printf("---PHASE B VOLTAGE LIST:\n");
                     for(j=0;j<COMM_IV_SIZE_R1;++j){
-                        printf("\t%g,\n", COMM.v_data_R1[j] );
+                        printf("\t%g,\n", COMM.v_phase_data_R1[j] );
                     }
                 #endif
+                /* 无意义 */
                 #if EXCITE_BETA_AXIS_AND_MEASURE_PHASE_B
-                    COMM.last_voltage_command = UQ_OUTPUT;    
+                    COMM.last_voltage_command = UQ_OUTPUT;
                 #else
                     COMM.last_voltage_command = UD_OUTPUT;
                 #endif
-                G.bool_comm_status = -1; // 由于此时电流是零，last_voltage_command也是零，就不能做电感辨识的开环激励辨识了，直接结束吧。
+                // 由于此时电流是零，last_voltage_command也是零，就不能做电感辨识的开环激励辨识了，直接结束吧。
+                COMM.bool_comm_status = -1; 
                 COMM.counterEntered = 0;
 
                 // UD_OUTPUT = 0.0;
@@ -569,8 +587,10 @@ void COMM_resistanceId_v2(REAL id_fb, REAL iq_fb){
         }
     }
 
-    #undef RS_ID_NUMBER_OF_STAIRS
-    #undef RS_ID_MAXIMUM_CURRENT 
+    #undef NOS
+    #undef NOS_II
+    #undef RS_ID_MAXIMUM_CURRENT
+    #undef RS_ID_LOW_CURRENT
     #undef REGULATOR
     #undef FEEDBACK
 }
@@ -643,7 +663,7 @@ void COMM_inductanceId(REAL id_fb, REAL iq_fb){
             printf("[Actual]  m=%g, b=%g, r=%g\n", m,b,r);
             printf("COMM.L = %g\n", COMM.L);
         #endif
-        G.bool_comm_status = 3;
+        COMM.bool_comm_status = 3;
         COMM.counterEntered = 0;
     }
 
@@ -714,7 +734,7 @@ void COMM_inductanceId_ver2(REAL id_fb, REAL iq_fb){
             #endif
             number_of_repeats_L2 += 1;
             if(number_of_repeats_L2 > 500){
-                G.bool_comm_status = 3;
+                COMM.bool_comm_status = 3;
                 number_of_repeats_L2 = 0;
             }
         }
@@ -807,8 +827,8 @@ void COMM_inductanceId_ver3(REAL id_fb, REAL iq_fb){
             printf("R3=%g, L3=%g, mztime=%g, triggertime=%g\n", COMM.R3, COMM.L3, mz_timebase, (1.0-HORY_MZ2014_SAMPLE_DUTY)*N_SAMPLE*CL_TS);
         #endif
 
-        G.bool_comm_status = 4; // PM flux id
-        G.bool_comm_status = 5; // inertia id
+        COMM.bool_comm_status = 4; // PM flux id
+        COMM.bool_comm_status = 5; // inertia id
         COMM.counterEntered = 0;
 
 
@@ -1014,7 +1034,7 @@ void COMM_PMFluxId(REAL id_fb, REAL iq_fb, REAL omg_elec_fb){
                 #endif
                 // getch();
 
-                G.bool_comm_status = 6; // end
+                COMM.bool_comm_status = 6; // end
                 COMM.counterEntered = 0;
 
                 accumKE = 0;
@@ -1145,8 +1165,8 @@ void COMM_inertiaId(REAL id_fb, REAL iq_fb, REAL cosPark, REAL sinPark, REAL omg
         sum_B = 0.0;
         COMM.number_of_repeats_Js += 1;
         if(COMM.number_of_repeats_Js>10){
-            G.bool_comm_status = 6;
-            G.bool_comm_status = 4; // PM flux id
+            COMM.bool_comm_status = 6;
+            COMM.bool_comm_status = 4; // PM flux id
 
             UD_OUTPUT = 0.0;
             UQ_OUTPUT = 0.0;
@@ -1185,17 +1205,17 @@ void COMM_end(REAL id_fb, REAL iq_fb){
 
 
 void StepByStepCommissioning(){
-    // G.bool_comm_status = 4;
+    // COMM.bool_comm_status = 4;
 
     /* Start *~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*/
 
     // 使用测量得到的电流、转速、位置等信息，生成电压指令CTRL.ual，CTRL.ube
     // commissioning();
-    if(G.bool_comm_status == 0){
+    if(COMM.bool_comm_status == 0){
         // 初始化
         init_COMM();
 
-    }else if(G.bool_comm_status == 1){
+    }else if(COMM.bool_comm_status == 1){
         // 电阻辨识
         if(G.flag_do_inverter_characteristics){
             COMM_resistanceId_v2(IS_C(0), IS_C(1)); // v2 is intended only for better inverter nonlinearity identification considering the rotor movement issue (starting from negative maximal current value to force align)
@@ -1203,20 +1223,20 @@ void StepByStepCommissioning(){
             COMM_resistanceId(IS_C(0), IS_C(1));
         }
 
-    }else if(G.bool_comm_status == 2){
+    }else if(COMM.bool_comm_status == 2){
         // 电感辨识（阶跃）
         COMM_inductanceId(IS_C(0), IS_C(1));
 
         // 电感辨识（方波）
         // COMM_inductanceId_ver2(IS_C(0), IS_C(1));
 
-    }else if(G.bool_comm_status == 3){
+    }else if(COMM.bool_comm_status == 3){
         // 电感辨识（正弦）
         COMM_inductanceId_ver3(IS_C(0), IS_C(1));
 
         // 更新电流PI
 
-    }else if(G.bool_comm_status == 4){
+    }else if(COMM.bool_comm_status == 4){
 
         // 永磁体磁链辨识
         CTRL.S->cosT = cos(CTRL.I->theta_d_elec);
@@ -1227,7 +1247,7 @@ void StepByStepCommissioning(){
 
         // 更新转速PI
 
-    }else if(G.bool_comm_status == 5){
+    }else if(COMM.bool_comm_status == 5){
 
         // 惯量辨识
         // CTRL.I->omg_elec     = EXP.omg_elec;
@@ -1246,7 +1266,7 @@ void StepByStepCommissioning(){
 
     }else{
         COMM_end(IS_C(0), IS_C(1));
-        G.bool_comm_status = -1;
+        COMM.bool_comm_status = -1;
     }
 
     /* End *~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*/      
