@@ -4,8 +4,22 @@
 
 // The function name for the dynamics
 #define MACHINE_DYNAMICS SM_Dynamics
-// id, iq, omega, theta_d, theta_d_accum
-#define NUMBER_OF_STATES 5
+// id, iq, omega, theta_d, theta_d_accum, stator flux alpha, stator flux beta
+#define NUMBER_OF_STATES 7
+
+#define INDEX_CURRENT_D 0
+#define INDEX_CURRENT_Q 1
+#define INDEX_OMEGA_ELEC 2
+#define INDEX_THETA_D 3 // elec
+#define INDEX_THETA_D_ACCUM 4 // mech
+#define INDEX_THETA_STATOR_FLUX_ALPHA 5 // in alpha-beta frame
+#define INDEX_THETA_STATOR_FLUX_BETA 6 // in alpha-beta frame
+
+#define INDEX_CURRENT_D 0
+#define INDEX_CURRENT_Q 1
+#define INDEX_OMEGA_ELEC 2
+#define INDEX_THETA_D 3 // elec
+#define INDEX_THETA_D_ACCUM 4 // mech
 
 struct SynchronousMachineSimulated{
 
@@ -47,13 +61,19 @@ struct SynchronousMachineSimulated{
     double dist_al;
     double dist_be;
 
-    double theta_d;
-    double theta_d_accum;
+    double theta_d; // elec
+    double theta_d_accum; // mech
+    double theta_d_accum_increment;
     double ud;
     double uq;
     double id;
     double iq;
 
+    // active flux
+    double psi_stator[2];
+    double psi_active[2];
+    double theta_d_activeFlux;
+
     // phase quantities
     double ua;
     double ub;
@@ -68,7 +88,7 @@ struct SynchronousMachineSimulated{
 extern struct SynchronousMachineSimulated ACM;
 
 extern int machine_simulation();
-extern void Machine_init();
+extern void init_Machine();
 
 #endif
 #endif
\ No newline at end of file
